use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007004
use Test::Spelling 0.12;
use Pod::Wordlist;

set_spell_cmd('aspell list');
add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Aas
Accomazzi
Adam
Alders
Alex
Alexandre
Alexey
Andreas
Asplund
Atzberger
Authen
BGMNT
Basic
Beijen
Betts
Bill
Bochner
BooK
Bron
Bryan
Buenzli
Burke
CGI
CPAN
Campbell
Cardillo
Chamas
Chase
Christopher
Chrysostomos
ConnCache
Coppit
Couzins
DAVIDRW
Daina
Dalgleish
Daniel
David
Debug
DebugFile
Denaxas
Digest
Dubois
Dunkin
Durechova
Duret
Dvornik
Eldridge
Etheridge
FWILES
Father
Fesler
Finch
Froehlich
Gavin
Gerhard
Gertjan
Gisle
Gondwana
Graaff
Graeme
Graham
Greab
Gregoire
Grossmann
Guenther
Gurusamy
Gustafsson
Hakanson
Hanak
Hans
Harald
Hay
Hedlund
Hoblitt
Horsfall
Hukins
Hwa
INOUE
Ian
Jacob
Jakub
Jason
Jeremy
Joao
Joe
Joerg
John
Jr
KONISHI
Kaminsky
Kapranoff
Karaban
Karen
Kartik
Katarina
Katsuhiro
Kebsch
Keiichiro
Kennedy
Kilgore
Kilzer
Klar
Knop
Koenig
Koster
Kronengold
Krüger
Kubb
König
LWP
Laker
Langfeldt
Langheinrich
Liam
Lindley
Lipcon
Lotterer
Lutz
MacEachern
Macdonald
Madsen
Mailto
Mann
Mark
Marko
Markus
Martijn
Massjouni
Mates
Matthew
Matyukhin
McCauley
Melchner
MemberMixin
Michael
Michiel
Mike
Moritz
Moshe
Murrell
NNTP
NTLM
Nagano
Naveed
Newby
Nicolai
Nierstrasz
Nigel
Ntlm
Olaf
Olly
Ondrej
Onken
Oosten
Panchenko
Peter
Peters
Pettit
Pimlott
Piotr
Pon
Poul
Protocol
Quaranta
Rabbitson
Radoslaw
Radu
Rai
Rezic
Robert
RobotUA
Rolf
Roszatycki
Sarathy
Schilli
Schinder
Schwern
Sean
Shirazi
Simple
Sjogren
Skyttä
Slaven
Spafford
Spiros
Steffen
Steffen_Ullrich
Steinbrunner
Steve
SteveHay
Stone
Stosberg
Subbarao
TCP
Takanori
Theodore
Thoennes
Thompson
Thurn
Tilly
Tim
Todd
Tom
Tony
Toru
Tourbin
TraceHTTP
UA
Ugai
Ullrich
Unger
UserAgent
VanHeyningen
Vandewege
Ville
Vyacheslav
Wheeler
Whitener
Wilk
WireShark
Wittkoski
Yamaguchi
Yee
Yitzchak
Yoshinari
Yuri
Yury
Zajac
Zakharevich
Zavarin
Zefram
Zielinski
Zoest
adamk
afPuUsSedvhx
amir
amire80
andreas
asjo
at
bin
brong
bryan
capoeirab
cjm
cpan
data
david
davidrw
de
denaxas
dot
dpettit
dsteinbrunner
erik
ether
file
freeerider
ftp
gerhard
gisle
github
gong
gopher
gpeters
haarg
hfroehlich
http
iank
jefflee
jfesler
jmates
john9art
jwilk
jwittkoski
ka
katarina
lib
loopback
lwp
mailto
mark
michael
michealgong
michiel
mmcleric
murphy
naveedm9
nigelgregoire
nntp
nogo
ohiocore
olaf
ondrej
onken
peterm
phrstbrn
piotr
rg
ribasushi
ruff
sasao
sburke
schwern
shaohua
shildreth
slaven
sprout
talby
tech
tim
todd
tom
trcjr
turugina
uid39246
ville
waif
wfmann
wolfsage
yury
zefram
zigorou
